<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Store\Model\ScopeInterface;

class RedirectResponseBefore implements ObserverInterface {

    protected $_helper;
    protected $redirectAllowed = false;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    public function __construct(\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig, \Magento\Framework\UrlInterface $urlBuilder, \Magento\Store\Model\StoreManagerInterface $storeManager, \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper
    ) {
        $this->_helper = $helper;
        $this->scopeConfig = $scopeConfig;
        $this->urlBuilder = $urlBuilder;
        $this->storeManager = $storeManager;
    }

    public function execute(\Magento\Framework\Event\Observer $observer) {

        $request = $observer->getRequest();
        $response = $observer->getResponse();

        $currentStoreId = $this->storeManager->getStore()->getId();
        $scopeStores = ScopeInterface::SCOPE_STORES;

        $this->redirectAllowed = $this->_helper->applyLogic($request, false, $this->urlBuilder->getCurrentUrl(), null);

        if ($this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId) && $this->redirectAllowed) {
            // Set header to prevent varnish caching
            $response->setHeader('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0', true);
            $response->setHeader('Pragma', 'no-cache', true);

            if (!$this->_helper->geolocationRequest($request)) {
                $head = '<script>window.filoblu_geoip_redirect_allowed = true;</script>';
                $response->setBody(preg_replace('/<\/head>/', $head . '</head>', $response->getBody(), 1));
            }
        }
    }

}
